# chrono - agendas system
# Copyright (C) 2022  Entr'ouvert
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django import forms
from django.forms import ValidationError
from django.template import Template, TemplateSyntaxError
from django.utils.translation import ugettext_lazy as _

from chrono.pricing.models import Criteria, CriteriaCategory


class NewCriteriaForm(forms.ModelForm):
    class Meta:
        model = Criteria
        fields = ['label', 'condition']

    def clean_condition(self):
        condition = self.cleaned_data['condition']
        try:
            Template('{%% if %s %%}OK{%% endif %%}' % condition)
        except TemplateSyntaxError:
            raise ValidationError(_('Invalid syntax.'))

        return condition


class CriteriaForm(NewCriteriaForm):
    class Meta:
        model = Criteria
        fields = ['label', 'slug', 'condition']

    def clean_slug(self):
        slug = self.cleaned_data['slug']

        if self.instance.category.criterias.filter(slug=slug).exclude(pk=self.instance.pk).exists():
            raise ValidationError(_('Another criteria exists with the same identifier.'))

        return slug


class PricingVariableForm(forms.Form):
    key = forms.CharField(label=_('Variable name'), required=False)
    value = forms.CharField(
        label=_('Value template'), widget=forms.TextInput(attrs={'size': 60}), required=False
    )


PricingVariableFormSet = forms.formset_factory(PricingVariableForm)


class PricingCriteriaCategoryAddForm(forms.Form):
    category = forms.ModelChoiceField(
        label=_('Criteria category to add'), queryset=CriteriaCategory.objects.none(), required=True
    )

    def __init__(self, *args, **kwargs):
        self.pricing = kwargs.pop('pricing')
        super().__init__(*args, **kwargs)
        self.fields['category'].queryset = CriteriaCategory.objects.exclude(pricings=self.pricing)


class PricingCriteriaCategoryEditForm(forms.Form):
    criterias = forms.ModelMultipleChoiceField(
        label=_('Criterias'),
        queryset=Criteria.objects.none(),
        required=True,
        widget=forms.CheckboxSelectMultiple(),
    )

    def __init__(self, *args, **kwargs):
        self.pricing = kwargs.pop('pricing')
        self.category = kwargs.pop('category')
        super().__init__(*args, **kwargs)
        self.fields['criterias'].queryset = self.category.criterias.all()
        self.initial['criterias'] = self.pricing.criterias.filter(category=self.category)
